use std::fmt::Display;

use serde::Serialize;
use sqlx::postgres::{PgHasArrayType, PgTypeInfo};

use crate::utils::ApplicationError;

#[derive(sqlx::Type, Debug, Clone, Copy, PartialEq, Eq, Serialize, PartialOrd, Ord)]
#[sqlx(type_name = "function", rename_all = "lowercase")]
pub enum Function {
    Posten = 1,
    Fuehrungsassistent = 5,
    Wachhabender = 10,
}

#[derive(sqlx::Type, Debug, Clone, PartialEq, Eq, Serialize, PartialOrd, Ord)]
#[sqlx(no_pg_array)]
pub struct Functions(Vec<Function>);

//impl sqlx::Type<Postgres> for Functions {
//    fn type_info() -> sqlx::postgres::PgTypeInfo {
//        // Array type name is the name of the element type prefixed with `_`
//        sqlx::postgres::PgTypeInfo::with_name("_functions")
//    }
//}

impl PgHasArrayType for Functions {
    fn array_type_info() -> PgTypeInfo {
        PgTypeInfo::with_name("function[]")
    }
}

impl Display for Functions {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let mut iterator = self.0.iter().peekable();
        while let Some(p) = iterator.next() {
            write!(f, "{}", p.to_string());
            if iterator.peek().is_some() {
                write!(f, ", ");
            }
        }
        todo!()
    }
}

impl Functions {
    pub fn contains(&self, f: &Function) -> bool {
        self.0.contains(f)
    }

    pub fn is_posten(&self) -> bool {
        self.0.contains(&Function::Posten)
    }

    pub fn is_fuehrungsassistent(&self) -> bool {
        self.0.contains(&Function::Fuehrungsassistent)
    }

    pub fn is_wachhabender(&self) -> bool {
        self.0.contains(&Function::Wachhabender)
    }
}

impl Display for Function {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Function::Posten => write!(f, "Posten"),
            Function::Fuehrungsassistent => write!(f, "Führungsassistent"),
            Function::Wachhabender => write!(f, "Wachhabender"),
        }
    }
}

impl TryFrom<u8> for Function {
    type Error = ApplicationError;

    fn try_from(value: u8) -> Result<Self, Self::Error> {
        match value {
            1 => Ok(Function::Posten),
            5 => Ok(Function::Fuehrungsassistent),
            10 => Ok(Function::Wachhabender),
            _ => Err(ApplicationError::UnsupportedEnumValue {
                value: value.to_string(),
                enum_name: String::from("Function"),
            }),
        }
    }
}

impl Default for Function {
    fn default() -> Self {
        Self::Posten
    }
}
